<?php

declare(strict_types=1);

namespace Drupal\multilingual_state;

use Drupal\Core\Language\LanguageDefault;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\State\StateInterface;

/**
 * Provides the state system using a key value store.
 */
class MultilingualState implements StateInterface {

  /**
   * Constructs a MultilingualState object.
   */
  public function __construct(
    protected readonly StateInterface $state,
    protected readonly LanguageManagerInterface $languageManager,
    protected readonly LanguageDefault $languageDefault,
  ) {}

  /**
   * {@inheritdoc}
   */
  public function get($key, $default = NULL) {
    $langcode = $this->languageManager->getCurrentLanguage()->getId();
    return $this->getTranslated($key, $langcode, $default);
  }

  /**
   * {@inheritdoc}
   */
  public function getMultiple(array $keys) {
    $values = [];
    foreach ($keys as $key) {
      $values[$key] = $this->get($key);
    }

    return $values;
  }

  /**
   * {@inheritdoc}
   */
  public function set($key, $value) {
    $currentLangcode = $this->languageManager->getCurrentLanguage()->getId();
    $this->setTranslated($key, $value, $currentLangcode);
  }

  /**
   * {@inheritdoc}
   */
  public function setMultiple(array $data) {
    foreach ($data as $key => $value) {
      $this->set($key, $value);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function delete($key) {
    $suffix = '__' . $this->languageManager->getCurrentLanguage()->getId();
    $this->state->delete($key . $suffix);
  }

  /**
   * {@inheritdoc}
   */
  public function deleteMultiple(array $keys) {
    foreach ($keys as $key) {
      $this->delete($key);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function resetCache() {
    $this->state->resetCache();
  }

  /**
   * Get a variable with a language suffix.
   */
  public function getTranslated($key, $langcode, $default = NULL) {
    $suffix = '__' . $langcode;

    $value = $this->state->get($key . $suffix, $default);
    if ($value === NULL) {
      return $this->state->get($key, $default);
    }
    return $value;
  }

  /**
   * Set a variable with a language suffix.
   */
  public function setTranslated($key, $value, $langcode) {
    $suffix = '__' . $langcode;
    $this->state->set($key . $suffix, $value);

    // When the current language is also the default store it prefix-less
    // as a fallback.
    if ($langcode === $this->languageDefault->get()->getId()) {
      $this->state->set($key, $value);

    }
  }

  /**
   * Delete the variable for all languages.
   */
  public function deleteAllLanguages($key) {
    foreach ($this->languageManager->getLanguages() as $language) {
      $suffix = '__' . $language->getId();
      $this->state->delete($key . $suffix);
    }
    $this->state->delete($key);
  }

}
