<?php

declare(strict_types=1);

namespace Drupal\Tests\multilingual_state\Unit;

use Drupal\Component\Datetime\TimeInterface;
use Drupal\Core\Cache\MemoryBackend;
use Drupal\Core\KeyValueStore\KeyValueMemoryFactory;
use Drupal\Core\Language\LanguageDefault;
use Drupal\Core\Lock\NullLockBackend;
use Drupal\Core\State\State;
use Drupal\Tests\UnitTestCase;
use Drupal\multilingual_state\MultilingualState;

/**
 * Tests the MultilingualState class.
 *
 * @group Multilingual
 */
class MultilingualStateTest extends UnitTestCase {

  /**
   * An instance of the class for testing.
   *
   * @var \Drupal\multilingual_state\MultilingualState
   */
  protected $multilingualState;

  /**
   * The current state of the test in memory.
   *
   * @var \Drupal\Core\State\State
   */
  protected $state;

  /**
   * Languages installed.
   */
  protected array $languages;

  /**
   * The current state of the test in memory.
   *
   * @var \Drupal\Core\Language\LanguageInterface
   */
  protected $languageManager;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    // Construct a state object used for testing logger assertions.
    $time = $this->prophesize(TimeInterface::class)->reveal();
    $this->state = new State(new KeyValueMemoryFactory(), new MemoryBackend($time), new NullLockBackend());

    // Set up some languages.
    $languageEs = $this->createMock('\Drupal\Core\Language\LanguageInterface');
    $languageEs->expects($this->any())
      ->method('getId')
      ->willReturn('es');
    $languageEn = $this->createMock('\Drupal\Core\Language\LanguageInterface');
    $languageEn->expects($this->any())
      ->method('getId')
      ->willReturn('en');
    $languages = [
      'es' => $languageEs,
      'en' => $languageEn,
    ];
    $this->languages = $languages;

    // Create a language manager stub.
    $this->languageManager = $this->createMock('Drupal\language\ConfigurableLanguageManagerInterface');
    $this->languageManager->expects($this->any())
      ->method('getLanguages')
      ->willReturn($languages);

    // Control the current language with each call of set/get.
    $this->languageManager->expects($this->any())
      ->method('getCurrentLanguage')
      ->willReturnOnConsecutiveCalls(
        $this->languages['en'],
        $this->languages['en'],
        $this->languages['es'],
        $this->languages['en'],
      );

    $languageDefault = $this->createMock(LanguageDefault::class);
    $languageDefault->expects($this->any())
      ->method('get')
      ->willReturn($languages['en']);

    $this->multilingualState = new MultilingualState($this->state, $this->languageManager, $languageDefault);
  }

  /**
   * Test service methods.
   */
  public function testTranslate(): void {
    $this->multilingualState->set('up', 'north');
    $this->assertEquals($this->multilingualState->get('up'), 'north');
    $this->multilingualState->set('up', 'norte');
    // The English value is still there.
    $this->assertEquals($this->multilingualState->get('up'), 'north');
    // Check the case of setting a variable and setting its fallback when
    // dealing with the default language.
    $this->multilingualState->setTranslated('down', 'south', 'en');
    // Request Spanish and get the fallback.
    $this->assertEquals($this->multilingualState->getTranslated('down', 'es'), 'south');
    $this->multilingualState->setTranslated('down', 'sur', 'es');
    // Request the same and get the translated value.
    $this->assertEquals($this->multilingualState->getTranslated('down', 'es'), 'sur');

    $this->multilingualState->deleteAllLanguages('down');
    $this->assertNull($this->multilingualState->getTranslated('down', 'es'));
  }

}
